@@ -46,6 +46,7 @@ Tokens *Lexer::tokenize(char *script)
 	char ch = smgr->currentChar();
 	for (; ch != EOL; smgr->idx++) {
 		ch = smgr->currentChar();
+		if (smgr->end()) break;
 		if (ch == '\n') ctx->finfo.start_line_num++;
 		if (scanner.isSkip(ctx)) {
 			continue;
@@ -97,6 +98,21 @@ Tokens *Lexer::tokenize(char *script)
 			smgr->idx += ctx->progress;
 			ctx->progress = 0;
 			break;
+		case 'x': {
+			char next_ch = smgr->nextChar();
+			char after_next_ch = smgr->afterNextChar();
+			if (next_ch != '=' || ctx->existsBuffer() ||
+				after_next_ch == '=' ||
+				after_next_ch == '>' ||
+				after_next_ch == '~') {
+				ctx->writeBuffer(ch);
+			} else {
+				tmgr->add(scanner.scanSymbol(ctx));
+				smgr->idx += ctx->progress;
+				ctx->progress = 0;
+			}
+			break;
+		}
 		case '\n':
 			tmgr->add(scanner.scanLineDelimiter(ctx));
 			tmgr->add(scanner.scanWhiteSpace(ctx)); // For newline character
@@ -229,14 +245,14 @@ void Lexer::prepare(Tokens *tokens)
 		Token *t = ITER_CAST(Token *, it);
 		switch (t->info.type) {
 		case TokenType::HereDocumentTag: case TokenType::HereDocumentRawTag:
-		case TokenType::HereDocumentExecTag:
+		case TokenType::HereDocumentExecTag: case TokenType::HereDocumentBareTag:
 			tag_pos = it;
 			break;
 		case TokenType::HereDocument: {
 			assert(tag_pos != start_pos && "ERROR!: nothing use HereDocumentTag");
 			Token *tag = ITER_CAST(Token *, tag_pos);
 			switch (tag->info.type) {
-			case TokenType::HereDocumentTag:
+				case TokenType::HereDocumentTag: case TokenType::HereDocumentBareTag:
 				tag->info.type = Enum::Token::Type::RegDoubleQuote;
 				tag->info.kind = Enum::Token::Kind::RegPrefix;
 				tag->info.name = "RegDoubleQuote";
